@@ -1,6 +1,7 @@
 import datetime
 
 import htmlgenerator as hg
+from django.contrib.staticfiles.storage import staticfiles_storage
 from django.db import models
 from django.shortcuts import get_object_or_404
 from django.urls import path
@@ -11,6 +12,7 @@ from bread import formatters
 from bread import layout as _layout
 from bread import menu, views
 from bread.utils import filter_fieldlist, generate_excel, urls, xlsxresponse
+from bread.utils.links import Link
 from bread.views.browse import delete
 from bread.views.edit import bulkcopy
 
@@ -22,6 +24,12 @@ class EditView(views.EditView):
     def get_layout(self):
         F = _layout.form.FormField
         ret = hg.BaseElement(
+            hg.LINK(
+                rel="stylesheet",
+                type="text/css",
+                href=staticfiles_storage.url("djangoql/css/completion.css"),
+            ),
+            hg.SCRIPT(src=staticfiles_storage.url("djangoql/js/completion.js")),
             hg.H3(self.object),
             _layout.form.Form.wrap_with_form(
                 hg.C("form"),
@@ -99,11 +107,10 @@ class ReadView(views.ReadView):
             title=self.object.name,
             helper_text=f"{self.object.queryset.count()} {self.object.model.model_class()._meta.verbose_name_plural}",
             primary_button=_layout.button.Button.fromaction(
-                menu.Link.from_objectaction(
-                    self.object,
-                    "excel",
+                Link(
+                    urls.reverse_model(self.object, "excel", {"pk": self.object.pk}),
                     label=_("Excel"),
-                    icon="download",
+                    iconname="download",
                 )
             ),
         )
@@ -141,13 +148,14 @@ urlpatterns = [
             columns=["name", "created"],
             rowclickaction="read",
             bulkactions=[
-                (
-                    menu.Link("delete", label=_("Delete"), icon="trash-can"),
-                    delete,
+                views.browse.BulkAction(
+                    "delete", label=_("Delete"), iconname="trash-can", action=delete
                 ),
-                (
-                    menu.Link("copy", label=_("Copy"), icon="copy"),
-                    lambda request, qs: bulkcopy(
+                views.browse.BulkAction(
+                    "copy",
+                    label=_("Copy"),
+                    iconname="copy",
+                    action=lambda request, qs: bulkcopy(
                         request, qs, labelfield="name", copy_related_fields=("columns",)
                     ),
                 ),
@@ -156,23 +164,23 @@ urlpatterns = [
                 menu.Action(
                     js=hg.BaseElement(
                         "document.location = '",
-                        hg.F(lambda c, e: _layout.objectaction(c["row"], "edit")),
+                        hg.F(lambda c: _layout.objectaction(c["row"], "edit")),
                         "'",
                     ),
-                    icon="edit",
+                    iconname="edit",
                     label=_("Edit"),
                 ),
                 menu.Action(
                     js=hg.BaseElement(
                         "document.location = '",
                         hg.F(
-                            lambda c, e: urls.reverse_model(
+                            lambda c: urls.reverse_model(
                                 Report, "excel", kwargs={"pk": c["row"].pk}
                             )
                         ),
                         "'",
                     ),
-                    icon="download",
+                    iconname="download",
                     label=_("Excel"),
                 ),
             ],
@@ -186,7 +194,7 @@ urlpatterns = [
         editview=EditView,
         readview=ReadView,
     ),
-    urls.generate_path(
+    urls.autopath(
         exceldownload,
         urls.model_urlname(Report, "excel"),
     ),
@@ -199,7 +207,11 @@ urlpatterns = [
 
 menu.registeritem(
     menu.Item(
-        menu.Link(urls.reverse_model(Report, "browse"), label=_("Reports")),
-        menu.Group(_("Reports"), icon="download"),
+        Link(
+            urls.reverse_model(Report, "browse"),
+            label=_("Reports"),
+            iconname="download",
+        ),
+        menu.Group(_("Reports"), iconname="download"),
     )
 )
