@@ -4,8 +4,9 @@ import htmlgenerator as hg
 from django.db import models
 from django.utils.translation import gettext_lazy as _
 
-from bread.menu import Action, Link
+from bread.menu import Action
 from bread.utils import filter_fieldlist, pretty_modelname, resolve_modellookup
+from bread.utils.links import Link
 from bread.utils.urls import link_with_urlparameters, reverse_model
 
 from ..base import aslink_attributes, fieldlabel, objectaction
@@ -22,10 +23,17 @@ class DataTableColumn(NamedTuple):
     cell: Any
     sortingname: Optional[str] = None
     enable_row_click: bool = True
+    th_attributes: hg.F = None
+    td_attributes: hg.F = None
 
     @staticmethod
     def from_modelfield(
-        col, model, prevent_automatic_sortingnames=False, rowvariable="row"
+        col,
+        model,
+        prevent_automatic_sortingnames=False,
+        rowvariable="row",
+        th_attributes=None,
+        td_attributes=None,
     ) -> "DataTableColumn":
         return DataTableColumn(
             fieldlabel(model, col),
@@ -33,6 +41,8 @@ class DataTableColumn(NamedTuple):
             sortingname_for_column(model, col)
             if not prevent_automatic_sortingnames
             else None,
+            th_attributes=th_attributes,
+            td_attributes=td_attributes,
         )
 
     def as_header_cell(self, orderingurlparameter="ordering"):
@@ -54,7 +64,7 @@ class DataTableColumn(NamedTuple):
                 title=self.header,
                 **sortinglink_for_column(orderingurlparameter, self.sortingname),
             )
-        return hg.TH(headcontent, **getattr(self.header, "td_attributes", {}))
+        return hg.TH(headcontent, lazy_attributes=self.th_attributes)
 
 
 class DataTable(hg.TABLE):
@@ -130,12 +140,12 @@ class DataTable(hg.TABLE):
                 action = Action(
                     js=hg.BaseElement(
                         "submitbulkaction(this.closest('[data-table]'), '",
-                        action.url,
+                        action.href,
                         f"', method='{getattr(action, 'method', 'GET')}')",
                     ),
                     label=action.label,
-                    icon=action.icon,
-                    permissions=action._permissions,
+                    iconname=action.iconname,
+                    permissions=action.permissions,
                 )
                 bulkactionlist.append(Button.fromaction(action))
             elif isinstance(action, Action):
@@ -177,9 +187,7 @@ class DataTable(hg.TABLE):
                         name=checkbox_for_bulkaction_name,
                         value=hg.If(
                             hg.F(
-                                lambda c, e: hasattr(
-                                    c[self.iterator.loopvariable], "pk"
-                                )
+                                lambda c: hasattr(c[self.iterator.loopvariable], "pk")
                             ),
                             hg.C(f"{self.iterator.loopvariable}.pk"),
                             hg.C(f"{self.iterator.loopvariable}_index"),
@@ -345,11 +353,11 @@ class DataTable(hg.TABLE):
 
         if addurl is None:
             addurl = hg.F(
-                lambda c, e: reverse_model(
+                lambda c: reverse_model(
                     model,
                     "add",
                     query={
-                        "next": hg.resolve_lazy(backurl, c, e)
+                        "next": hg.resolve_lazy(backurl, c)
                         or c["request"].get_full_path()
                     },
                 )
@@ -367,7 +375,7 @@ class DataTable(hg.TABLE):
                     rowactions,
                     "action",
                     hg.F(
-                        lambda c, e: Button.fromaction(
+                        lambda c: Button.fromaction(
                             c["action"],
                             notext=True,
                             small=True,
@@ -379,42 +387,52 @@ class DataTable(hg.TABLE):
                 style="display: flex",
             )
 
-        action_menu_header = hg.BaseElement()
-        action_menu_header.td_attributes = {"_class": "bx--table-column-menu"}
         queryset = model.objects.all() if queryset is None else queryset
         columns = columns or filter_fieldlist(model, ["__all__"])
         column_definitions: List[DataTableColumn] = []
         for col in columns:
+            td_attributes = None
+            if rowclickaction and getattr(col, "enable_row_click", True):
+                td_attributes = hg.F(
+                    lambda c: aslink_attributes(
+                        objectaction(c[rowvariable], rowclickaction)
+                    )
+                )
             # convert simple string (modelfield) to column definition
             if isinstance(col, str):
                 col = DataTableColumn.from_modelfield(
-                    col, model, prevent_automatic_sortingnames, rowvariable
+                    col,
+                    model,
+                    prevent_automatic_sortingnames,
+                    rowvariable,
+                    td_attributes=td_attributes,
                 )
+            else:
+                if td_attributes:
+                    col = col._replace(td_attributes=td_attributes)
 
-            if rowclickaction and col.enable_row_click:
-                col.cell.td_attributes = aslink_attributes(
-                    hg.F(
-                        lambda c, e: objectaction(
-                            c[rowvariable],
-                            rowclickaction,
-                            query={
-                                "next": hg.resolve_lazy(backurl, c, e)
-                                or c["request"].get_full_path()
-                            },
-                        )
-                    )
-                )
             column_definitions.append(col)
 
         table = DataTable(
             column_definitions
             + (
-                [DataTableColumn(action_menu_header, objectactions_menu)]
+                [
+                    DataTableColumn(
+                        "",
+                        objectactions_menu,
+                        td_attributes=hg.F(
+                            lambda c: {"_class": "bx--table-column-menu"}
+                        ),
+                        th_attributes=hg.F(
+                            lambda c: {"_class": "bx--table-column-menu"}
+                        ),
+                    )
+                ]
                 if rowactions
                 else []
             ),
             # querysets are cached, the call to all will make sure a new query is used in every request
-            hg.F(lambda c, e: queryset),
+            hg.F(lambda c: queryset),
             **kwargs,
         )
         if with_toolbar:
@@ -460,10 +478,7 @@ class DataTable(hg.TABLE):
     def row(columns):
         """Returns a row element based on the specified columns"""
         return hg.TR(
-            *[
-                hg.TD(col.cell, **getattr(col.cell, "td_attributes", {}))
-                for col in columns
-            ]
+            *[hg.TD(col.cell, lazy_attributes=col.td_attributes) for col in columns]
         )
 
     @staticmethod
@@ -481,7 +496,7 @@ class DataTable(hg.TABLE):
 
 
 def sortingclass_for_column(orderingurlparameter, columnname):
-    def extracturlparameter(context, element):
+    def extracturlparameter(context):
         value = context["request"].GET.get(orderingurlparameter, "")
         if not value:
             return ""
@@ -513,7 +528,7 @@ def sortinglink_for_column(orderingurlparameter, columnname):
         "-" + columnname: None,
     }
 
-    def extractsortinglink(context, element):
+    def extractsortinglink(context):
         currentordering = context["request"].GET.get(orderingurlparameter, None)
         nextordering = ORDERING_VALUES.get(currentordering, columnname)
         return link_with_urlparameters(
