@@ -1,9 +1,11 @@
+from typing import Callable, NamedTuple, Optional
+
 import htmlgenerator as hg
 from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth.mixins import LoginRequiredMixin
 from django.db import models
-from django.http import HttpResponse, HttpResponseRedirect
+from django.http import HttpRequest, HttpResponse
 from django.shortcuts import redirect
 from django.utils.translation import gettext_lazy as _
 from django.views.generic import ListView
@@ -14,20 +16,35 @@ from bread.utils import expand_ALL_constant, filter_fieldlist
 
 from .. import layout as _layout  # prevent name clashing
 from ..layout.base import fieldlabel
-from ..menu import Link
 from ..utils import (
     generate_excel,
     link_with_urlparameters,
     pretty_modelname,
     xlsxresponse,
 )
+from ..utils.links import Link
 from .util import BreadView
 
 
+class BulkAction(NamedTuple):
+    name: str
+    label: str
+    action: Callable[[HttpRequest, models.query.QuerySet], Optional[HttpResponse]]
+    iconname: str = "fade"
+    permissions: list[str] = []
+
+    def has_permission(self, request, obj=None):
+        return all(
+            [
+                request.user.has_perm(perm, obj) or request.user.has_perm(perm)
+                for perm in self.permissions
+            ]
+        )
+
+
 class BrowseView(BreadView, LoginRequiredMixin, PermissionListMixin, ListView):
     """TODO: documentation"""
 
-    template_name = "bread/base.html"
     orderingurlparameter = "ordering"
     itemsperpage_urlparameter = "itemsperpage"
     objectids_urlparameter = "_selected"  # see bread/static/js/main.js:submitbulkaction and bread/layout/components/datatable.py
@@ -40,9 +57,10 @@ class BrowseView(BreadView, LoginRequiredMixin, PermissionListMixin, ListView):
     # bulkactions: List[(Link, function(request, queryset))]
     # - link.js should be a slug and not a URL
     # - if the function returns a HttpResponse, the response is returned instead of the browse view result
-    bulkactions = []
+    bulkactions = ()
     rowactions = ()  # list of links
     backurl = None
+    addurl = None
 
     def __init__(self, *args, **kwargs):
         self.orderingurlparameter = (
@@ -72,21 +90,23 @@ class BrowseView(BreadView, LoginRequiredMixin, PermissionListMixin, ListView):
         )
         self.rowclickaction = kwargs.get("rowclickaction") or self.rowclickaction
         self.backurl = kwargs.get("backurl") or self.backurl
+        self.addurl = kwargs.get("addurl") or self.addurl
         super().__init__(*args, **kwargs)
         # set some default bulkactions
         self.bulkactions = (
             kwargs.get("bulkactions")
             or self.bulkactions
-            or [
-                (
-                    Link("excel", label=_("Excel"), icon="download"),
-                    lambda request, qs: export(qs, self.columns),
+            or (
+                BulkAction(
+                    "excel",
+                    label=_("Excel"),
+                    iconname="download",
+                    action=lambda request, qs: export(qs, self.columns),
                 ),
-                (
-                    Link("delete", label=_("Delete"), icon="trash-can"),
-                    delete,
+                BulkAction(
+                    "delete", label=_("Delete"), iconname="trash-can", action=delete
                 ),
-            ]
+            )
         )
 
     def get_layout(self):
@@ -97,12 +117,12 @@ class BrowseView(BreadView, LoginRequiredMixin, PermissionListMixin, ListView):
         bulkactions = [
             Link(
                 link_with_urlparameters(
-                    self.request, **{self.bulkaction_urlparameter: action.url}
+                    self.request, **{self.bulkaction_urlparameter: action.name}
                 ),
                 label=action.label,
-                icon=action.icon,
+                iconname=action.iconname,
             )
-            for action, _ in self.bulkactions
+            for action in self.bulkactions
             if action.has_permission(self.request)
         ]
         return _layout.datatable.DataTable.from_model(
@@ -121,6 +141,7 @@ class BrowseView(BreadView, LoginRequiredMixin, PermissionListMixin, ListView):
             checkbox_for_bulkaction_name=self.objectids_urlparameter,
             settingspanel=self.get_settingspanel(),
             backurl=self.backurl,
+            addurl=self.addurl,
         )
 
     def get_context_data(self, *args, **kwargs):
@@ -141,8 +162,8 @@ class BrowseView(BreadView, LoginRequiredMixin, PermissionListMixin, ListView):
             return redirect(self.request.path)
         if self.bulkaction_urlparameter in self.request.GET:
             bulkactions = {
-                action.url: actionfunction
-                for action, actionfunction in self.bulkactions
+                action.name: action.action
+                for action in self.bulkactions
                 if action.has_permission(self.request)
             }
             if self.request.GET[self.bulkaction_urlparameter] not in bulkactions:
@@ -152,11 +173,15 @@ class BrowseView(BreadView, LoginRequiredMixin, PermissionListMixin, ListView):
                     % self.request.GET[self.bulkaction_urlparameter],
                 )
             else:
-                result = bulkactions[self.request.GET[self.bulkaction_urlparameter]](
+                ret = bulkactions[self.request.GET[self.bulkaction_urlparameter]](
                     self.request, self.get_queryset()
                 )
-                if isinstance(result, HttpResponse):
-                    return result
+                params = self.request.GET.copy()
+                del params[self.bulkaction_urlparameter]
+                del params[self.objectids_urlparameter]
+                if ret is None:
+                    return redirect(self.request.path + "?" + params.urlencode())
+                return ret
         return super().get(*args, **kwargs)
 
     def get_paginate_by(self, queryset):
@@ -241,11 +266,13 @@ def delete(request, queryset, softdeletefield=None, required_permissions=None):
                     _("Your user has not the permissions to delete %s") % instance
                 )
             if softdeletefield:
-                setattr(instance, softdeletefield, True)
-                instance.save()
+                if not getattr(instance, softdeletefield):
+                    setattr(instance, softdeletefield, True)
+                    instance.save()
+                    deleted += 1
             else:
                 instance.delete()
-            deleted += 1
+                deleted += 1
         except Exception as e:
             messages.error(
                 request,
@@ -260,4 +287,39 @@ def delete(request, queryset, softdeletefield=None, required_permissions=None):
             "modelname": pretty_modelname(queryset.model, plural=deleted > 1),
         },
     )
-    return HttpResponseRedirect(redirect_to=request.path)
+
+
+def restore(request, queryset, softdeletefield, required_permissions=None):
+    if required_permissions is None:
+        required_permissions = [
+            f"{queryset.model._meta.app_label}.change_{queryset.model.__name__.lower()}"
+        ]
+
+    restored = 0
+    for instance in queryset:
+        try:
+            if not request.user.has_perm(required_permissions, instance):
+                # we throw an exception here because the user not supposed to
+                # see the option to restore an object anyway, if he does not have the permssions
+                # the queryset should already be filtered
+                raise Exception(
+                    _("Your user has not the permissions to restore %s") % instance
+                )
+            if getattr(instance, softdeletefield, False):
+                setattr(instance, softdeletefield, False)
+                instance.save()
+                restored += 1
+        except Exception as e:
+            messages.error(
+                request,
+                _("%s could not be restored: %s") % (object, e),
+            )
+
+    messages.success(
+        request,
+        _("Restored %(count)s %(modelname)s")
+        % {
+            "count": restored,
+            "modelname": pretty_modelname(queryset.model, plural=restored > 1),
+        },
+    )
