@@ -12,7 +12,6 @@ from .edit import EditView
 class ReadView(EditView):
     """TODO: documentation"""
 
-    template_name = "bread/base.html"
     accept_global_perms = True
     fields = None
     urlparams = (("pk", int),)
@@ -47,8 +46,6 @@ class ReadView(EditView):
 
 
 def layoutasreadonly(layout):
-    if getattr(layout, "readonly", False):
-        return layout
     layout.wrap(
         lambda element, ancestors: isinstance(element, _layout.form.Form)
         and element.standalone,
@@ -69,5 +66,4 @@ def layoutasreadonly(layout):
     )
     for form in layout.filter(lambda element, ancestors: isinstance(element, hg.FORM)):
         form.attributes["onsubmit"] = "return false;"
-    layout.readonly = True
     return layout
