@@ -1,13 +1,17 @@
+import typing
 import urllib
 
 import htmlgenerator as hg
 from django import forms
+from django.conf import settings
 from django.contrib import messages
+from django.http import HttpResponse
 from django.utils.html import mark_safe
+from django.utils.module_loading import import_string
 
-from .. import layout as _layout  # prevent name clashing
+from .. import layout as breadlayout
+from .. import menu
 from ..forms.forms import breadmodelform_factory
-from ..layout.components.form import FormField
 from ..utils import filter_fieldlist, reverse_model
 
 
@@ -38,12 +42,12 @@ class CustomFormMixin:
         ret = hg.BaseElement()
         for field in self.fields or formfields:
             if field in formfields:
-                ret.append(_layout.form.FormField(field))
+                ret.append(breadlayout.form.FormField(field))
             else:
                 ret.append(field)
         return hg.BaseElement(
             hg.H3(self.object),
-            _layout.form.Form.wrap_with_form(hg.C("form"), ret),
+            breadlayout.form.Form.wrap_with_form(hg.C("form"), ret),
         )
 
     def get_form(self, form_class=None):
@@ -52,7 +56,9 @@ class CustomFormMixin:
         # hide or disable predefined fields passed in GET parameters
         if self.request.method != "POST":
             for fieldelement in self._get_layout_cached().filter(
-                lambda element, ancestors: isinstance(element, FormField)
+                lambda element, ancestors: isinstance(
+                    element, breadlayout.form.FormField
+                )
             ):
                 if (
                     fieldelement.fieldname in self.request.GET
@@ -77,6 +83,17 @@ class CustomFormMixin:
                 )
         return form
 
+    def form_valid(self, *args, **kwargs):
+        ret = super().form_valid(*args, **kwargs)
+        if self.ajax_urlparameter in self.request.GET:
+            ret = HttpResponse("OK")
+            # This header will be processed by htmx
+            # in order to reload the whole page automatically
+            # (instead of doing the redirect which is required for
+            # normal POST submission responses
+            ret["HX-Refresh"] = "true"
+        return ret
+
     def get_success_url(self):
         if self.request.GET.get("next"):
             return urllib.parse.unquote(self.request.GET["next"])
@@ -88,25 +105,52 @@ class BreadView:
     Shortcut to create a subclass with the given attributes
     """
 
-    layout = None
-    _layout = None
-
-    @classmethod
-    def __init_subclass__(cls, **kwargs):
-        super().__init_subclass__(**kwargs)
+    layout: hg.BaseElement = None
+    _layout_cached: hg.BaseElement = None
+    ajax_urlparameter = "asajax"
+    page_layout: typing.Callable[[menu.Menu, hg.BaseElement], hg.BaseElement] = None
 
     @classmethod
     def _with(cls, **kwargs):
+        """
+        Helper which allows to quickly create a parameterized view without needing to
+        implement a new class in user code. E.g. BrowseView.with(fields=["name", "email"])
+        """
         return type(f"Custom{cls.__name__}", (cls,), kwargs)
 
-    def _get_layout_cached(self):
-        """Used for caching layouts, only bread-internal"""
-        if self._layout is None:
-            self._layout = self.get_layout()
-        return self._layout
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.ajax_urlparameter = kwargs.get(
+            "ajax_urlparameter", getattr(self, "ajax_urlparameter")
+        )
+
+        self.page_layout = (
+            kwargs.get("page_layout", getattr(self, "page_layout"))
+            or settings.DEFAULT_PAGE_LAYOUT
+        )
+        if isinstance(self.page_layout, str):
+            self.page_layout = import_string(self.page_layout)
+
+    def get_page_layout(self, menu, content_layout):
+        return self.page_layout(menu, content_layout)
 
     def get_layout(self):
-        """Returns the layout for this view, returns the ``layout`` attribute by default"""
+        """Returns the layout for this view, returns the ``layout`` attribute by default.
+        Either set the ``layout`` attribute or override this method."""
         if self.layout is None:
             raise RuntimeError(f"'layout' of view {self} is None")
         return self.layout
+
+    def render_to_response(self, context, **response_kwargs):
+        response_kwargs.setdefault("content_type", self.content_type)
+        ret = self._get_layout_cached()
+        if self.ajax_urlparameter not in self.request.GET:
+            ret = self.get_page_layout(menu.main, ret)
+
+        return breadlayout.render(self.request, ret, context, **response_kwargs)
+
+    def _get_layout_cached(self):
+        """Used for caching layouts, only bread-internal"""
+        if self._layout_cached is None:
+            self._layout_cached = self.get_layout()
+        return self._layout_cached
